#!/usr/bin/perl -w
#
# $Id: preasm_patcher.pl,v 1.3 2012-02-20 09:02:11 vrsieh Exp $
#
# Copyright (C) 2005-2009 FAUmachine Team <info@faumachine.org>.
# This program is free software. You can redistribute it and/or modify it
# under the terms of the GNU General Public License, either version 2 of
# the License, or (at your option) any later version. See COPYING.

#FIXME JOSEF: take care that no .text.* get section patched too

$DEBUG = 0;
$PROGRAM = "preasm_patcher.pl";

$GEN_HEAD = "\t/* Generated by $PROGRAM */	\n";
$GEN_TAIL = "\t/* End of $PROGRAM */		\n";
$in_file = $ARGV[0];
$out_file = $ARGV[1];
$out_file2 = "_$ARGV[1]";
# when specified, sections beginning with default names .text, .data .rodata*
# get prefixed with that name

$section_prefix = $ARGV[2];

@def_sec_names = ('.text','.data','.rodata');

sub check_section_spec
{

	my ($section);
	my ($line);
	($line) = @_;

	# ".section" keyword
	if ($line =~ /^\s*\.section\b\s*([\.\w]*)/) {
		$section = $1;
	
	# ".text" keyword
	} elsif ($line =~ /^\s*\.text\s*/) {
		$section = ".text";
	
	# ".data" keyword
	} elsif ($line =~ /^\s*\.data\s*/) {;
		$section = ".data";

	# No section specifier
	} else {
		$section = undef;
	}
	return $section;
}

sub get_var
{
	$_ = $_[0];
	s/^(.)(.)(\w*)(\s)(.*)(\n)$/$5/;

	if ($DEBUG) {
		print "get_var: $_[0] -> $_\n";
	}

	return $_;
}

sub change_varname
{
	# Searches for $var in $line. If founds then adds
	# prefix $PM into $var in $line
	if (!defined($var)) {
		return $_[0];
	}

	$_ = $_[0];
	if (/$_[1]/) {
		$pm = "PM";
		$varstart = index($line,$_[1],0);
		$varlen = length($_[1]);
		$str = substr($_[0], 0, $varstart);
		$str2 = substr($_[0], $varstart+$varlen, length($_[0])-($varstart+$varlen));
		if ($DEBUG) {
			print "start:$str\n";
			print "end:$str2";
		}
		$_ = "$str$pm$_[1]$str2";
		if ($DEBUG) {
			print "line($varstart):$_";
		}
	}

	return $_;
}

	$section_prefix = undef;

open(INFILE, $in_file) 
	|| die "$PROGRAM: unable to open input file \"$in_file\" ($!)";

my ($cur_sect);
$cur_sect = ".text";
my %local_labels = ();

# First pass: collect necessary data 
while (defined($line = <INFILE>)) {
	if (defined(check_section_spec($line))) {
		$cur_sect =  check_section_spec($line);
	} elsif ($line =~ /\s*(.*):.*/) {
		$local_labels{$1} = [$cur_sect];
	}# Nothing todo any more 
}

# Second pass: patch it
seek(INFILE, 0, 0);
open(OUT, ">$out_file") 
	|| die "$PROGRAM: unable to open output file \"$out_file\" ($!)";

@patched_func_bodies = ();
$code_size = 32;
$cur_sect = ".text";

# Patch ASM output of assert and fixme C files (pm)
$patch = 0;
if ($in_file =~ /librt_pm.*[a|f][s|i][s|x]/ and $patch == 1) {
	open(OUT2, ">$out_file2") 
		|| die "$PROGRAM: unable to open output file \"$out_file2\" ($!)";
	print "\n--> uprava souboru: $in_file <--\n\n"; # !!!
	$prefix = "PM";
	$var = undef;
	while (defined($line = <INFILE>)) {
		if ($line =~ /.globl __.*/ and (not defined($var))) {
			if ($DEBUG) {
				print "$line";
			}
			$var = get_var($line);
			if ($DEBUG) {
				print "$var\n";
			}
			print OUT2 "\t.globl $prefix$var\n";
			
		} else {
			$line = change_varname($line, $var);
			print OUT2 "$line";
		}
	}

	close(INFILE) 
		|| die "$PROGRAM: unable to close input file \"$in_file\" ($!)";
	close(OUT2) 
		|| die "$PROGRAM: unable to close output file \"$out_file2\" ($!)";
	$in_file = $out_file2;
	open(INFILE, $in_file) 
		|| die "$PROGRAM: unable to open input file \"$in_file\" ($!)";
}

while (defined($line = <INFILE>)) {
	
	# Detect 32-bit code
	if ($line =~ /\s*\.code32\b\s*/) {
		if ($DEBUG) {
			print OUT "/* $PROGRAM: Begin of 32-bit code */\n";
		}
		$code_size = 32;
	} elsif ($line =~ /\s*\.code16(|gcc)\b\s*/) {
		if ($code_size eq 32) {
			$code_size = 16;
			if ($DEBUG) {
				print OUT "/* $PROGRAM: End of 32-bit code */\n";
			}
		}
	}

	#Detect current section
	if (defined(check_section_spec($line))) {
		$cur_sect = check_section_spec($line);
		if ($DEBUG) {
			print OUT "/* $PROGRAM: Begin of section: $cur_sect */\n";
		}
	}

	## Patch default section names
	if (defined($section_prefix) and defined(check_section_spec($line))) {

		## .section pseudo opcode
		$new_line = undef;
		foreach $s (@def_sec_names) {
			$_ = $line;
			# REMEMBER: from whitespace to "." is no \b pattern
			s/\s*\.section\b\s*${s}/\t.section ${section_prefix}${s}/;
			if ($_ ne $line) {
				$new_line = $_;
				last;
			}
		}

		## .data and .text pseudo opcode
		if (not defined($new_line)) {
			$_ = $line;
			s/^\s*\.text\s*/\t.section ${section_prefix}.text, "ax", \@progbits\n/;
			s/^\s*\.data\s*/\t.section ${section_prefix}.data, "ax", \@progbits\n/;
			$new_line = $_;
		}

		## Write patched line
		if ($new_line ne $line) {
			if ($DEBUG) {
				print OUT $GEN_HEAD, "\t/* section patch */	\n";
				## FIXME JOSEF: prettier
				print OUT "/* old section spec: $line */	\n";
				print OUT $GEN_TAIL;
			}
			print OUT $new_line;
		} else {
			print OUT $line; 
		}

# Nothing to patch
	} else {
		print OUT "$line";
	}
}

close(INFILE) 
	|| die "$PROGRAM: unable to close input file \"$in_file\" ($!)";
close(OUT) 
	|| die "$PROGRAM: unable to close output file \"$out_file\" ($!)";


if ($DEBUG) {
	$all_bodies = join(",", @patched_func_bodies);
	print "patched function bodies: $all_bodies\n"
}
